@@ -8,7 +8,7 @@ use SVK::I18N;
 use SVK::Editor::Delay;
 use SVK::Command::Log;
 use SVK::Merge;
-use SVK::Util qw (get_buffer_from_editor find_svm_source svn_mirror);
+use SVK::Util qw( get_buffer_from_editor find_svm_source );
 
 sub options {
     ($_[0]->SUPER::options,
@@ -81,7 +81,7 @@ sub run {
 	($dst->{report}, '', 1) if $dst->{copath};
 
     if ($self->{incremental} && !$self->{check_only}) {
-	die loc ("Not possible to do incremental merge without merge ticket.\n")
+	die loc ("Not possible to do incremental merge without a merge ticket.\n")
 	    if $self->{no_ticket};
 	print loc ("-m ignored in incremental merge\n") if $self->{message};
 	my @rev;
@@ -123,21 +123,20 @@ SVK::Command::Merge - Apply differences between two sources
 
 =head1 SYNOPSIS
 
-merge -r N:M DEPOTPATH [PATH]\r
-merge -r N:M DEPOTPATH1 DEPOTPATH2
+ merge -r N:M DEPOTPATH [PATH]
+ merge -r N:M DEPOTPATH1 DEPOTPATH2
 
 =head1 OPTIONS
 
- -r [--revision] rev:       revision
- -m [--message] message:    commit message
- -C [--check-only]:         don't perform actual writes
- -I [--incremental]:        apply changes individually
- -a [--auto]:               automatically find merge points
- -l [--log]:                brings the logs of merged revs to the message buffer
- --no-ticket:               don't associate the ticket tracking merge history
- --track-rename:            track the changes made to renamed node
- --force:                   Needs description
- -s [--sign]:               Needs description
+ -r [--revision] N:M    : act on revisions between N and M
+ -m [--message] arg     : specify commit message ARG
+ -C [--check-only]      : try operation but make no changes
+ -I [--incremental]     : apply each change individually
+ -a [--auto]            : merge from the previous merge point
+ -l [--log]             : use logs of merged revisions as commit message
+ -s [--sign]            : sign this change
+ --no-ticket            : do not record this merge point
+ --track-rename         : track changes made to renamed node
 
 =head1 AUTHORS
 
