@@ -3,8 +3,8 @@ use strict;
 our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command::Commit );
-use SVK::Util qw(svn_mirror);
 use SVK::I18N;
+use SVK::Util qw( HAS_SVN_MIRROR );
 
 sub options {
     ('upgrade' => 'upgrade',
@@ -15,7 +15,7 @@ sub parse_arg {
     my ($self, $path, @arg) = @_;
 
     # Allow "svk mi uri://... //depot" to mean "svk mi //depot uri://"
-    if (@arg and $path =~ m{^\w+:}) {
+    if (@arg and $path =~ /^[A-Za-z][-+.A-Za-z0-9]*:/) {
 	($arg[0], $path) = ($path, $arg[0]);
     }
 
@@ -27,7 +27,7 @@ sub lock { $_[0]->lock_none }
 
 sub run {
     my ($self, $target, $source, @options) = @_;
-    die loc("cannot load SVN::Mirror") unless svn_mirror;
+    die loc("cannot load SVN::Mirror") unless HAS_SVN_MIRROR;
 
     if ($self->{upgrade}) {
 	SVN::Mirror::upgrade ($target->{repos});
@@ -41,12 +41,12 @@ sub run {
 	my $fmt = "%-20s %-s\n";
 	printf $fmt, 'Path', 'Source';
 	print '=' x 60;
+	my ($depot) = ($target->{depotpath} =~ /^(\/\w*)/);
 	for (@paths) {
 	    my $m = SVN::Mirror->new (target_path => $_, repos => $target->{repos},
 				      get_source => 1);
-	    printf $fmt, $_, $m->{source};
+	    printf $fmt, $depot.$_, $m->{source};
 	}
-	print '=' x 60;
 	return;
     }
 
@@ -56,7 +56,7 @@ sub run {
 			      repos => $target->{repos},
 			      options => \@options,
 			      config => $self->{svnconfig},
-			      pool => SVN::Pool->new, auth => $self->auth,
+			      pool => SVN::Pool->new,
 			      # XXX: remove in next svn::mirror release
 			      target => $target->{repospath},
 			     );
@@ -75,20 +75,21 @@ SVK::Command::Mirror - Initialize a mirrored depotpath
 
 =head1 SYNOPSIS
 
- mirror [http|svn]://server.host/path DEPOTPATH
+ mirror [http|svn]://host/path DEPOTPATH
  mirror cvs::pserver:user@host:/cvsroot:module/... DEPOTPATH
- mirror p4:user@host:1666://depot/module/... DEPOTPATH
+ mirror p4:user@host:1666://path/... DEPOTPATH
 
  # You may also list the target part first:
- mirror DEPOTPATH [http|svn]://server.host/path
+ mirror DEPOTPATH [http|svn]://host/path
 
  mirror --list
+ mirror --upgrade //
  mirror --upgrade /DEPOT/
 
 =head1 OPTIONS
 
- --list:               List mirrored path
- --upgrade:            Upgrade mirror state to latest version
+ --list                 : list mirrored paths
+ --upgrade              : upgrade mirror state to the latest version
 
 =head1 AUTHORS
 
