@@ -8,7 +8,7 @@ use SVK::XD;
 sub options {
     ($_[0]->SUPER::options,
      'B|baseless'	=> 'baseless',
-     'b|base:i'		=> 'base',
+     'b|base|baserev:i'	=> 'baserev',
     );
 }
 
@@ -33,17 +33,17 @@ SVK::Command::Smerge - Automatic merge all changes between branches
 
 =head1 OPTIONS
 
- -m [--message] message:    commit message
- -C [--check-only]:         don't perform actual writes
- -I [--incremental]:        apply changes individually
- -l [--log]:                brings the logs of merged revs to the message buffer
- --remoterev:               Use remote revision in merge log
- --host host:               Use host as hostname shown in merge log
- --no-ticket:               don't associate the ticket tracking merge history
- -B [--baseless]:           use the oldest revision as the merge point
- -b [--base] rev:           manually specify source revision as the merge point
- --force:                   Needs description
- -s [--sign]:               Needs description
+ -m [--message] arg     : specify commit message ARG
+ -C [--check-only]      : try operation but make no changes
+ -I [--incremental]     : apply each change individually
+ -l [--log]             : use logs of merged revisions as commit message
+ -B [--baseless]        : use the earliest revision as the merge point
+ -b [--base] arg        : use revision ARG as the merge point
+ -s [--sign]            : sign this change
+ --no-ticket            : do not record this merge point
+ --track-rename         : track changes made to renamed node
+ --host arg             : use ARG as the hostname shown in merge log
+ --remoterev            : use remote revision numbers in merge log
 
 =head1 AUTHORS
 
