@@ -9,7 +9,7 @@ use File::Spec;
 
 sub parse_arg {
     my ($self, @arg) = @_;
-    $self->usage if $#arg < 0 || $#arg > 1;
+    return if $#arg < 0 || $#arg > 1;
     my $depotpath = $self->arg_depotpath ($arg[0]);
     return ($depotpath, $self->arg_copath ($arg[1] || ''));
 }
@@ -21,7 +21,8 @@ sub run {
     die loc("different depot") unless $target->same_repos ($cotarget);
 
     my ($entry, @where) = $self->{xd}{checkout}->get ($cotarget->{copath});
-    die loc("can only switch checkout root") unless $where[0] eq $cotarget->{copath};
+    die loc("Can only switch checkout root.\n")
+	unless $where[0] eq $cotarget->{copath};
 
     $self->{update_target_path} = $target->{path};
     # check if the switch has a base at all
@@ -40,7 +41,7 @@ __DATA__
 
 =head1 NAME
 
-SVK::Command::Switch - Switch to another branch and keep local modifications
+SVK::Command::Switch - Switch to another branch and keep local changes
 
 =head1 SYNOPSIS
 
@@ -48,7 +49,7 @@ SVK::Command::Switch - Switch to another branch and keep local modifications
 
 =head1 OPTIONS
 
- -r [--revision]:        revision
+ -r [--revision] arg    : act on revision ARG instead of the head revision
 
 =head1 AUTHORS
 
