@@ -4,6 +4,7 @@ our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command::Commit );
 use SVK::I18N;
+use SVK::Util qw( HAS_SVN_MIRROR );
 
 sub options {
     ('s|skipto=s'	=> 'skip_to',
@@ -26,7 +27,7 @@ sub copy_notify {
 
 sub run {
     my ($self, @arg) = @_;
-    die loc("cannot load SVN::Mirror") unless $self->svn_mirror;
+    die loc("cannot load SVN::Mirror") unless HAS_SVN_MIRROR;
 
     die loc("argument skipto not allowed when multiple target specified")
 	if $self->{skip_to} && ($self->{sync_all} || $#arg > 0);
@@ -45,7 +46,7 @@ sub run {
 	my $m = SVN::Mirror->new (target_path => $target->{path},
 				  target => $target->{repospath},
 				  repos => $target->{repos},
-				  pool => SVN::Pool->new, auth => $self->auth,
+				  pool => SVN::Pool->new,
 				  config => $self->{svnconfig},
 				  cb_copy_notify => \&copy_notify,
 				  revprop => ['svk:signature'],
@@ -67,12 +68,13 @@ SVK::Command::Sync - Synchronize a mirrored depotpath
 =head1 SYNOPSIS
 
  sync DEPOTPATH
+ sync -a
 
 =head1 OPTIONS
 
- -a [--all]:           Needs description
- -t [--torev] arg:     Needs description
- -s [--skipto] arg:    Needs description
+ -a [--all]             : synchronize all mirrored paths
+ -s [--skipto] arg      : start synchronization at revision ARG
+ -t [--torev] arg       : stop synchronization at revision ARG
 
 =head1 AUTHORS
 
