@@ -1,5 +1,7 @@
 package SVK::Patch;
 use strict;
+use SVK::I18N;
+use SVK::Util qw( read_file write_file );
 our $VERSION = $SVK::VERSION;
 
 =head1 NAME
@@ -73,9 +75,7 @@ Load a SVK::Patch object from file.
 
 sub load {
     my ($class, $file, $xd, $depot) = @_;
-    open FH, '<', $file or die $!;
-    local $/;
-    my $self = thaw (uncompress (decode_base64(<FH>)));
+    my $self = thaw (uncompress (decode_base64 ( read_file($file) ) ) );
     $self->{_xd} = $xd;
     $self->{_depot} = $depot;
     for (qw/source target/) {
@@ -98,8 +98,7 @@ Store a SVK::Patch object to file.
 sub store {
     my ($self, $file) = @_;
     my $store = bless {map {m/^_/ ? () : ($_ => $self->{$_})} keys %$self}, ref ($self);
-    open FH, '>', $file or die $!;
-    print FH encode_base64(compress (nfreeze ($store)));
+    write_file( $file, encode_base64(compress (nfreeze ($store))) );
 }
 
 =head2 editor
@@ -126,8 +125,12 @@ sub _path_attribute_text {
 	    ++$mirrored;
 	}
     }
-    return ($local ? ' [local]' : '').($mirrored ? ' [mirrored]' : '').
-	($self->{"_${type}_updated"} ? ' [updated]' : '');
+    return join(
+	' ', '',
+	($local ? '[local]' : ()),
+	($mirrored ? '[mirrored]' : ()),
+	($self->{"_${type}_updated"} ? '[updated]' : ())
+    );
 }
 
 sub view {
@@ -140,7 +143,7 @@ sub view {
 	$self->_path_attribute_text ('target')."\n";
     print "Log:\n".$self->{log}."\n";
 
-    die "Target not local nor mirrored, unable to view patch."
+    die loc("Target not local nor mirrored, unable to view patch.\n")
 	unless $self->{_target};
 
     my $baseroot = $self->{_target}->root;
@@ -172,7 +175,7 @@ sub apply {
 sub apply_to {
     my ($self, $target, $storage, %cb) = @_;
     my $base = $self->{_target}
-	or die "Target not local nor mirrored, unable to test patch.";
+	or die loc("Target not local nor mirrored, unable to test patch.\n");
     # XXX: cb_merged
     my $editor = SVK::Editor::Merge->new
 	( base_anchor => $base->path,
@@ -191,7 +194,7 @@ sub apply_to {
 # XXX: update and regen are identify.  the only difference is soruce or target to be updated
 sub update {
     my ($self) = @_;
-    die "Target not local nor mirrored, unable to update patch."
+    die loc("Target not local nor mirrored, unable to update patch.\n")
 	unless $self->{_target};
 
     return unless $self->{_target_updated};
@@ -217,7 +220,7 @@ sub update {
 sub regen {
     my ($self) = @_;
     my $target = $self->{_target}
-	or die "Target not local nor mirrored, unable to regen patch.";
+	or die loc("Target not local nor mirrored, unable to regen patch.\n");
     unless ($self->{level} == 0 || $self->{_source_updated}) {
 	print "Source of path <$self->{name}> not updated or not local. No need to update.\n";
 	return;
