@@ -8,8 +8,10 @@ require SVN::Core;
 require SVN::Repos;
 require SVN::Fs;
 use File::Path;
+use File::Temp;
 use SVK;
 use SVK::XD;
+use SVK::Util qw( catdir tmpdir abs_path $SEP $EOL IS_WIN32 );
 use strict;
 use Carp;
 
@@ -25,11 +27,15 @@ our $copath;
 for (qw/SVKMERGE SVKDIFF LC_CTYPE LC_ALL LANG LC_MESSAGES/) {
     $ENV{$_} = '' if $ENV{$_};
 }
+$ENV{LANGUAGE} = $ENV{LANGUAGES} = 'i-default';
+
+# Make "prove -l" happy
+@INC = map abs_path($_), @INC;
 
 my $pool = SVN::Pool->new_default;
 
 sub new_repos {
-    my $repospath = "/tmp/svk-$$";
+    my $repospath = catdir(tmpdir(), "svk-$$");
     my $reposbase = $repospath;
     my $repos;
     my $i = 0;
@@ -37,8 +43,9 @@ sub new_repos {
 	$repospath = $reposbase . '-'. (++$i);
     }
     my $pool = SVN::Pool->new_default;
+    $ENV{SVNFSTYPE} ||= (($SVN::Core::VERSION =~ /^1\.0/) ? 'bdb' : 'fsfs');
     $repos = SVN::Repos::create("$repospath", undef, undef, undef,
-				{'fs-type' => $ENV{SVNFSTYPE} || 'bdb'})
+				{'fs-type' => $ENV{SVNFSTYPE}})
 	or die "failed to create repository at $repospath";
     return $repospath;
 }
@@ -49,7 +56,7 @@ sub build_test {
     my $depotmap = {map {$_ => (new_repos())[0]} '',@depot};
     my $xd = SVK::XD->new (depotmap => $depotmap,
 			   svkpath => $depotmap->{''},
-			   checkout => Data::Hierarchy->new);
+			   checkout => Data::Hierarchy->new( sep => $SEP ));
     my $svk = SVK->new (xd => $xd, output => \$output);
     push @TOCLEAN, [$xd, $svk];
     return ($xd, $svk);
@@ -87,23 +94,39 @@ sub cleanup_test {
 
 sub append_file {
     my ($file, $content) = @_;
-    open my ($fh), '>>:raw', $file or die "can't append $file: $!";
+    open my ($fh), '>>', $file or die "can't append $file: $!";
     print $fh $content;
     close $fh;
 }
 
 sub overwrite_file {
     my ($file, $content) = @_;
-    open my ($fh), '>:raw', $file or confess $!;
+    open my ($fh), '>', $file or confess "Cannot overwrite $file: $!";
+    print $fh $content;
+    close $fh;
+}
+
+sub overwrite_file_raw {
+    my ($file, $content) = @_;
+    open my ($fh), '>:raw', $file or confess "Cannot overwrite $file: $!";
     print $fh $content;
     close $fh;
 }
 
 sub is_file_content {
     my ($file, $content, $test) = @_;
-    open my ($fh), '<:raw', $file or confess $!;
+    open my ($fh), '<', $file or confess "Cannot read from $file: $!";
     local $/;
-    is (<$fh>, $content, $test);
+    @_ = (<$fh>, $content, $test);
+    goto &is;
+}
+
+sub is_file_content_raw {
+    my ($file, $content, $test) = @_;
+    open my ($fh), '<:raw', $file or confess "Cannot read from $file: $!";
+    local $/;
+    @_ = (<$fh>, $content, $test);
+    goto &is;
 }
 
 sub is_output {
@@ -111,8 +134,8 @@ sub is_output {
     $svk->$cmd (@$arg);
     my $cmp = (grep {ref ($_) eq 'Regexp'} @$expected)
 	? \&is_deeply_like : \&is_deeply;
-    $cmp->([split ("\n", $output)], $expected,
-	   $test || join(' ', $cmd, @$arg));
+    @_ = ([split (/\r?\n/, $output)], $expected, $test || join(' ', $cmd, @$arg));
+    goto &$cmp;
 }
 
 sub is_deeply_like {
@@ -121,25 +144,27 @@ sub is_deeply_like {
 	if (ref ($expected->[$_]) eq 'Regexp' ) {
 	    unless ($got->[$_] =~ m/$expected->[$_]/) {
 		diag "Different at $_:\n$got->[$_]";
-		ok (0, $test);
-		return;
+		@_ = (0, $test);
+		goto &ok;
 	    }
 	}
 	else {
 	    if ($got->[$_] ne $expected->[$_]) {
 		diag "Different at $_:\n$got->[$_]\n$expected->[$_]";
-		ok (0, $test);
-		return;
+		@_ = (0, $test);
+		goto &ok;
 	    }
 	}
     }
-    is ($#{$expected}, $#{$got}, $test);
+    @_ = ($#{$expected}, $#{$got}, $test);
+    goto &is;
 }
 
 sub is_output_like {
     my ($svk, $cmd, $arg, $expected, $test) = @_;
     $svk->$cmd (@$arg);
-    like ($output, $expected, $test || join(' ', $cmd, @$arg));
+    @_ = ($output, $expected, $test || join(' ', $cmd, @$arg));
+    goto &like;
 }
 
 sub copath {
@@ -156,6 +181,14 @@ sub status_native {
     return @ret;
 }
 
+sub status {
+    my @ret;
+    while (my ($status, $path) = splice (@_, 0, 2)) {
+	push @ret, join (' ', $status, $path);
+    }
+    return @ret;
+}
+
 require SVN::Simple::Edit;
 
 sub get_editor {
@@ -177,24 +210,25 @@ sub create_basic_tree {
     my $pool = SVN::Pool->new_default;
     my ($repospath, $path, $repos) = $xd->find_repos ($depot, 1);
 
+    local $/ = $EOL;
     my $edit = get_editor ($repospath, $path, $repos);
     $edit->open_root ();
 
     $edit->modify_file ($edit->add_file ('/me'),
-			"first line in me\n2nd line in me\n");
+			"first line in me$/2nd line in me$/");
     $edit->modify_file ($edit->add_file ('/A/be'),
-			"\$Rev\$ \$Rev\$\n\$Revision\$\nfirst line in be\n2nd line in be\n");
+			"\$Rev\$ \$Rev\$$/\$Revision\$$/first line in be$/2nd line in be$/");
     $edit->change_file_prop ('/A/be', 'svn:keywords', 'Rev URL Revision');
     $edit->modify_file ($edit->add_file ('/A/P/pe'),
-			"first line in pe\n2nd line in pe\n");
+			"first line in pe$/2nd line in pe$/");
     $edit->add_directory ('/B');
     $edit->add_directory ('/C');
     $edit->add_directory ('/A/Q');
     $edit->change_dir_prop ('/A/Q', 'foo', 'prop on A/Q');
     $edit->modify_file ($edit->add_file ('/A/Q/qu'),
-			"first line in qu\n2nd line in qu\n");
+			"first line in qu$/2nd line in qu$/");
     $edit->modify_file ($edit->add_file ('/A/Q/qz'),
-			"first line in qz\n2nd line in qz\n");
+			"first line in qz$/2nd line in qz$/");
     $edit->add_directory ('/C/R');
     $edit->close_edit ();
     my $tree = { child => { me => {},
@@ -211,13 +245,13 @@ sub create_basic_tree {
     my $rev = $repos->fs->youngest_rev;
     $edit = get_editor ($repospath, $path, $repos);
     $edit->open_root ();
-    $edit->modify_file ('/me', "first line in me\n2nd line in me - mod\n");
+    $edit->modify_file ('/me', "first line in me$/2nd line in me - mod$/");
     $edit->modify_file ($edit->add_file ('/B/fe'),
-			"file fe added later\n");
+			"file fe added later$/");
     $edit->delete_entry ('/A/P');
     $edit->copy_directory('/B/S', "file://${repospath}/${path}/A", $rev);
     $edit->modify_file ($edit->add_file ('/D/de'),
-			"file de added later\n");
+			"file de added later$/");
     $edit->close_edit ();
 
     $tree->{child}{B}{child}{fe} = {};
@@ -244,4 +278,45 @@ sub tree_from_xdroot {
     # generate a hash describing the content in an xdroot
 }
 
+sub __ ($) {
+    my $path = shift;
+    $path =~ s{/}{$SEP}go;
+    return $path;
+}
+
+sub _x { IS_WIN32 ? 1 : -x $_[0] }
+sub not_x { IS_WIN32 ? 1 : not -x $_[0] }
+sub _l { IS_WIN32 ? 1 : -l $_[0] }
+sub not_l { IS_WIN32 ? 1 : not -l $_[0] }
+
+sub uri {
+    my $file = shift;
+    $file =~ s{^|\\}{/}g if IS_WIN32;
+    return "file://$file";
+}
+
+my @unlink;
+sub set_editor {
+    my $tmp = File::Temp->new( SUFFIX => '.pl', UNLINK => 0 );
+    print $tmp $_[0];
+    $tmp->close;
+
+    my $perl = $^X;
+    my $tmpfile = $tmp->filename;
+
+    if (defined &Win32::GetShortPathName) {
+	$perl = Win32::GetShortPathName($perl);
+	$tmpfile = Win32::GetShortPathName($tmpfile);
+    }
+
+    chmod 0755, $tmpfile;
+    push @unlink, $tmpfile;
+
+    $ENV{SVN_EDITOR} = "$perl $tmpfile";
+}
+
+END {
+    unlink $_ for @unlink;
+}
+
 1;
